@@ -661,7 +661,7 @@ if (daemon_listen)
   union sockaddr_46 sin;
 
   #if HAVE_IPV6
-  struct in6_addr anyaddr6 = IN6ADDR_ANY_INIT;
+  struct in6_addr anyaddr6 = in6addr_any;
   #endif
 
   /* Find the standard SMTP port if no port number given; otherwise
@@ -689,15 +689,63 @@ if (daemon_listen)
       listen_socket_count++;
     }
 
-  /* Otherwise set up one address item with a null address, implying listening
-  on all interfaces. */
+  /* Otherwise we set up things to listen on all interfaces. In an IPv4 world,
+  this is just a single, empty address. On systems with IPv6, several different
+  implementation approaches have been taken. This code is now supposed to work
+  with all of them. The point of difference is whether an IPv6 socket that is
+  listening on all interfaces will receive incoming IPv4 calls or not.
+
+  . On Solaris, an IPv6 socket will accept IPv4 calls, and give them as mapped
+    addresses. However, if an IPv4 socket is also listening on all interfaces,
+    calls are directed to the appropriate socket.
+
+  . On (some versions of) Linux, an IPv6 socket will accept IPv4 calls, and
+    give them as mapped addresses, but an attempt also to listen on an IPv4
+    socket on all interfaces causes an error.
+
+  . On OpenBSD, an IPv6 socket will not accept IPv4 calls. You have to set up
+    two sockets if you want to accept both kinds of call.
+
+  . FreeBSD is like OpenBSD, but it has the IPV6_V6ONLY socket option, which
+    can be turned off, to make it behave like the versions of Linux described
+    above.
+
+  . I heard a report that the USAGI IPv6 stack for Linux has implemented
+    IPV6_V6ONLY.
+
+  So, what we do is as follows:
+
+   (1) At this point we set up two addresses, one containing ":" to indicate
+   an IPv6 wildcard address, and an empty one to indicate an IPv4 wildcard
+   address.
+
+   (2) Later, when we create the IPv6 socket, we set IPV6_V6ONLY if that option
+   is defined.
+
+   (3) We listen on the v6 socket first. If that fails, there is a serious
+   error.
+
+   (4) We listen on the v4 socket second. If that fails with the error
+   EADDRINUSE, assume we are in the situation where just a single socket is
+   permitted, and ignore the error. */
 
   else
     {
     addresses = store_get(sizeof(ip_address_item));
+
+    #if HAVE_IPV6
+    addresses->next = store_get(sizeof(ip_address_item));
+    addresses->address[0] = ':';
+    addresses->address[1] = 0;
+    addresses->next->next = NULL;
+    addresses->next->address[0] = 0;
+    listen_socket_count = 2;
+
+    #else
     addresses->next = NULL;
     addresses->address[0] = 0;
     listen_socket_count = 1;
+    #endif  /* HAVE_IPV6 */
     }
 
   /* Get a vector to remember all the sockets in */
@@ -714,41 +762,52 @@ if (daemon_listen)
   #endif
 
   /* For each IP address, create a socket and bind it to the appropriate
-  port. Note that if local_interfaces is not set, we bind a single socket to
-  all interfaces. */
+  port. See comments above about IPv6 sockets that may or may not accept IPv4
+  calls when listening on all interfaces. We also have to cope with the case of
+  a system with IPv6 libraries, but no IPv6 support in the kernel. In this
+  case, we must ignore failure to create an IPv6 socket for wildcard listening.
+  The second socket (IPv4) should then get used instead - we have to shuffle
+  it down into first place. */
 
   for (ipa = addresses, sk = 0; sk < listen_socket_count; ipa = ipa->next, sk++)
     {
     int i;
+    int af = (strchr(ipa->address, ':') != NULL)? AF_INET6 : AF_INET;
 
-    #if HAVE_IPV6
-    BOOL ipv6_socket = TRUE;
-    BOOL ipv6_address = strchr(ipa->address, ':') != NULL;
-
-    /* A system which has all the IPv6 libraries etc. may nevertheless not
-    support IPv6 in the kernel, so if the socket creation fails, fall back
-    on creating an IPv4 socket, except when we have an explicit IPv6 address,
-    for which we really do need an IPv6 socket. */
-
-    listen_sockets[sk] = socket(AF_INET6, SOCK_STREAM, 0);
+    listen_sockets[sk] = socket(af, SOCK_STREAM, 0);
     if (listen_sockets[sk] < 0)
       {
-      if (!ipv6_address)
+      /* Just log failure for an IPv6 wildcard socket */
+
+      if (af == AF_INET6 && local_interfaces == NULL)
         {
-        listen_sockets[sk] = socket(AF_INET, SOCK_STREAM, 0);
-        ipv6_socket = FALSE;
+        log_write(0, LOG_MAIN, "Failed to create IPv6 socket for wildcard "
+          "listening (%s): falling back to IPv4", strerror(errno));
+
+        addresses = addresses->next;   /* Chop IPv6 off the list */
+        sk--;                          /* Back up the count */
+        listen_socket_count--;         /* Reduce the total */
+        continue;                      /* With the IPv4 socket */
         }
+
+      /* Not a failure to create an IPv6 socket for wildcard listening */
+
+      else
+        log_write(0, LOG_PANIC_DIE, "IPv%c socket creation failed: %s",
+          (af == AF_INET6)? '6' : '4', strerror(errno));
       }
 
-    /* The IPv4 case */
+    /* If this is an IPv6 wildcard socket, set IPV6_V6ONLY if that option is
+    available. Just log failure (can get protocol not available, just like
+    socket creation can). */
 
-    #else    /* HAVE_IPV6 */
-    listen_sockets[sk] = socket(AF_INET, SOCK_STREAM, 0);
-    #endif   /* HAVE_IPV6 */
-
-    if (listen_sockets[sk] < 0)
-      log_write(0, LOG_PANIC_DIE, "socket creation failed: %s",
-        strerror(errno));
+    #ifdef IPV6_V6ONLY
+    if (local_interfaces == NULL && af == AF_INET6 &&
+        setsockopt(listen_sockets[sk], SOL_SOCKET, IPV6_V6ONLY, (char *)(&on),
+          sizeof(on)) < 0)
+      log_write(0, LOG_MAIN, "Setting IPV6_V6ONLY on daemon's IPv6 wildcard "
+        "socket failed (%s): carrying on without it", strerror(errno));
+    #endif  /* IPV6_V6ONLY */
 
     /* Set SO_REUSEADDR so that the daemon can be restarted while a connection
     is being handled.  Without this, a connection will prevent reuse of the
@@ -765,44 +824,31 @@ if (daemon_listen)
 
     memset(&sin, 0, sizeof(sin));
 
-    /* This code has to be entirely different for IPv6 and IPv4 because
-    of the different struct layouts. Sigh. Under IPv6, however, we may have
-    ended up with an IPv4 socket because the kernel may not support IPv6 even
-    if the libraries etc. are available. In this case, revert to IPv4 style
-    code (this will happen only when the the address is not an IPv6 one).
-    Although the code is identical, I've chosen to replicate it for ease of
-    reading the two cases. */
+    /* Setup code when using an IPv6 socket. The wildcard address is ":", to
+    ensure an IPv6 socket is used. */
 
     #if HAVE_IPV6
-    if (ipv6_socket)
+    if (af == AF_INET6)
       {
       sin.v6.sin6_family = AF_INET6;
       sin.v6.sin6_port = net_port;
-
-      if (ipa->address[0] == 0)
+      if (ipa->address[0] == ':' && ipa->address[1] == 0)
         {
         sin.v6.sin6_addr = anyaddr6;
         DEBUG(9) debug_printf("listening on all interfaces (IPv6)\n");
         }
       else
         {
-        int rc;
-        if (ipv6_address)
-          rc = inet_pton(AF_INET6, ipa->address, &sin.v6.sin6_addr);
-        else
-        /* We have an IPv6 socket, and an IPv4 address. We have to turn the
-        address into an IPv6 mapped form. */
-          {
-          memmove(ipa->address+7, ipa->address, (int)strlen(ipa->address) + 1);
-          memcpy(ipa->address, "::ffff:", 7);
-          rc = inet_pton(AF_INET6, ipa->address, &sin.v6.sin6_addr);
-          }
-        if (rc != 1)
+        if (inet_pton(AF_INET6, ipa->address, &sin.v6.sin6_addr) != 1)
           log_write(0, LOG_PANIC_DIE, "unable to parse \"%s\"", ipa->address);
         DEBUG(9) debug_printf("listening on %s\n", ipa->address);
         }
       }
-    else   /* Revert to IPv4 code */
+    else
+    #endif  /* HAVE_IPV6 */
+
+    /* Setup code when using IPv4 socket. The wildcard address is "". */
+
       {
       sin.v4.sin_family = AF_INET;
       sin.v4.sin_port = net_port;
@@ -817,34 +863,25 @@ if (daemon_listen)
         DEBUG(9) debug_printf("listening on %s\n", ipa->address);
         }
       }
-    #else   /* HAVE_IPV6 */
 
-
-    /* The IPv4 case is simple. */
-
-    sin.v4.sin_family = AF_INET;
-    sin.v4.sin_port = net_port;
-    if (ipa->address[0] == 0)
-      {
-      sin.v4.sin_addr.s_addr = (S_ADDR_TYPE)INADDR_ANY;
-      DEBUG(9) debug_printf("listening on all interfaces\n");
-      }
-    else
-      {
-      sin.v4.sin_addr.s_addr = (S_ADDR_TYPE)inet_addr(ipa->address);
-      DEBUG(9) debug_printf("listening on %s\n", ipa->address);
-      }
-    #endif  /* HAVE_IPV6 */
-
-
-    /* Retry loop */
+    /* Retry loop. Need to get the length of sin right for IPv4/IPv6
+    for some operating systems that check it. Although AF_INET6 is always
+    available (even on old IPv4 systems), the sin.v6 field isn't. */
 
     for (i = 9; i >= 0; i--)
       {
-      if (bind(listen_sockets[sk], (struct sockaddr *)&sin, sizeof(sin)) < 0)
+      if (bind(listen_sockets[sk], (struct sockaddr *)&sin,
+               #if HAVE_IPV6
+               (af == AF_INET6)? sizeof(sin.v6) : sizeof(sin.v4)
+               #else
+               sizeof(sin)
+               #endif /* HAVE_IPV6 */
+           ) < 0)
         {
         char *msg = strerror(errno);
-        char *addr = (ipa->address[0] == 0)? "(any)" : ipa->address;
+        char *addr = (ipa->address[0] == 0)? "(any IPv4)" :
+          (ipa->address[0] == ':' && ipa->address[1] == 0)? "(any IPv6)" :
+          ipa->address;
         if (i == 0)
           log_write(0, LOG_MAIN|LOG_PANIC_DIE,
             "socket bind() to port %d for address %s failed: %s: "
@@ -884,10 +921,35 @@ if (daemon_listen)
     }
 
   /* Start listening on the bound sockets, establishing the maximum backlog of
-  connections that is allowed. */
+  connections that is allowed. In an IPv6 environment, if listen() fails with
+  the error EADDRINUSE and we are doing wildcard listening and this is the
+  second (i.e last) socket, ignore the error on the grounds that we must be
+  in a system where the IPv6 socket accepts both kinds of call. */
 
   for (sk = 0; sk < listen_socket_count; sk++)
-    listen(listen_sockets[sk], smtp_connect_backlog);
+    {
+    if (listen(listen_sockets[sk], smtp_connect_backlog) < 0)
+      {
+      if (errno == EADDRINUSE && local_interfaces == NULL && sk > 0)
+        {
+        DEBUG(9) debug_printf("wildcard IPv4 listen() failed after IPv6 "
+          "listen() success; EADDRINUSE ignored\n");
+        close(listen_sockets[sk]);
+        listen_socket_count--;       /* OK because we know we are in the */
+        break;                       /* last iteration of this loop */
+        }
+      else
+        {
+        int skf;
+        for (ipa = addresses, skf = 0; skf < sk; ipa = ipa->next, skf++);
+        log_write(0, LOG_PANIC_DIE, "listen() failed on interface %s: %s",
+          (ipa->address[0] == 0)? "(any IPv4)" :
+          (ipa->address[0] == ':' && ipa->address[1] == ':')? "(any IPv6)" :
+          ipa->address,
+          strerror(errno));
+        }
+      }
+    }
   }
 
 /* Set up the handler for SIGHUP, which causes a restart of the daemon. */
@@ -1299,6 +1361,7 @@ for (;;)
     log_write(0, LOG_MAIN, "pid %d: SIGHUP received: re-exec daemon",
       getpid());
     for (sk = 0; sk < listen_socket_count; sk++) close(listen_sockets[sk]);
+    alarm(0);
     sighup_argv[0] = exim_path;
     execv(exim_path, sighup_argv);
     log_write(0, LOG_MAIN|LOG_PANIC_DIE, "pid %d: exec of %s failed: %s",
