@@ -174,6 +174,7 @@ static var_entry var_table[] = {
 
 static int var_table_size = sizeof(var_table)/sizeof(var_entry);
 static char var_buffer[256];
+static BOOL malformed_header;
 
 /* For textual hashes */
 
@@ -828,7 +829,7 @@ else if (strcmp(name, "exists") == 0
   struct stat statbuf;
   while (isspace((uschar)*s)) s++;
   if (*s != '{') goto COND_FAILED_CURLY_START;
-  sub = expand_string_internal(s+1, TRUE, &s, FALSE);
+  sub = expand_string_internal(s+1, TRUE, &s, yield == NULL);
   if (sub == NULL) return NULL;
   if (*s++ != '}') goto COND_FAILED_CURLY_END;
   if (yield != NULL)
@@ -974,7 +975,13 @@ else if (strcmp(name, "eq") == 0 ||
           "in \"%s\"", sub[1]);
         return NULL;
         }
-      *yield = (strcmp(crypt(sub[0], sub[1]), sub[1]) == 0) == testfor;
+
+      /* If the encrypted string contains fewer than two characters (for the
+      salt), force failure. Otherwise we get false positives: with an empty
+      string the yield of crypt() is an empty string! */
+
+      *yield = (strlen(sub[1]) < 2)? !testfor :
+        (strcmp(crypt(CS sub[0], CS sub[1]), sub[1]) == 0) == testfor;
       }
     break;
     #else
@@ -1067,6 +1074,7 @@ Arguments:
   sub        points to vector of pointers to set
   n          number of substrings
   sptr       points to current string pointer
+  skipping   the skipping flag
 
 Returns:     0 OK; string pointer updated
              1 curly bracketing error
@@ -1074,7 +1082,7 @@ Returns:     0 OK; string pointer updated
 */
 
 static int
-read_subs(char **sub, int n, char **sptr)
+read_subs(char **sub, int n, char **sptr, BOOL skipping)
 {
 int i;
 char *s = *sptr;
@@ -1083,7 +1091,7 @@ while (isspace((uschar)*s)) s++;
 for (i = 0; i < n; i++)
   {
   if (*s != '{') return 1;
-  sub[i] = expand_string_internal(s+1, TRUE, &s, FALSE);
+  sub[i] = expand_string_internal(s+1, TRUE, &s, skipping);
   if (sub[i] == NULL) return 2;
   while (isspace((uschar)*s)) s++;
   if (*s++ != '}') return 1;
@@ -1356,6 +1364,7 @@ Operators:
                            using a div/mod scheme to produce a string of the
                            form "a/b" where 0 <= a < n-1 and 0 <= b <= m-1.
                            If <m> is omitted, just give one number.
+   md5                     yields the md5 hash of the argument
    lc                      lowercase the string
    uc                      uppercase the string
    length_<n>              take first n chars only
@@ -1427,7 +1436,6 @@ int ptr = 0;
 int size = (int)strlen(string) + 64;
 char *s = string;
 char *yield = store_get(size);
-BOOL malformed_header = FALSE;
 char *save_expand_nstring[EXPAND_MAXN+1];
 int save_expand_nlength[EXPAND_MAXN+1];
 
@@ -1628,7 +1636,7 @@ while (*s != 0)
     while (isspace((uschar)*s)) s++;
     if (*s == '{')
       {
-      key = expand_string_internal(s+1, TRUE, &s, FALSE);
+      key = expand_string_internal(s+1, TRUE, &s, skipping);
       if (key == NULL) goto EXPAND_FAILED;
       if (*s++ != '}') goto EXPAND_FAILED_CURLY;
       while (isspace((uschar)*s)) s++;
@@ -1724,7 +1732,7 @@ while (*s != 0)
     single-key+file lookups, and the whole query otherwise. */
 
     if (*s != '{') goto EXPAND_FAILED_CURLY;
-    filename = expand_string_internal(s+1, TRUE, &s, FALSE);
+    filename = expand_string_internal(s+1, TRUE, &s, skipping);
     if (filename == NULL) goto EXPAND_FAILED;
     if (*s++ != '}') goto EXPAND_FAILED_CURLY;
     while (isspace((uschar)*s)) s++;
@@ -1818,7 +1826,7 @@ while (*s != 0)
 
     while (isspace((uschar)*s)) s++;
     if (*s != '{') goto EXPAND_FAILED_CURLY;
-    sub_name = expand_string_internal(s+1, TRUE, &s, FALSE);
+    sub_name = expand_string_internal(s+1, TRUE, &s, skipping);
     if (sub_name == NULL) goto EXPAND_FAILED;
     while (isspace((uschar)*s)) s++;
     if (*s++ != '}') goto EXPAND_FAILED_CURLY;
@@ -1834,7 +1842,7 @@ while (*s != 0)
             "(max is %d)", sub_name, EXIM_PERL_MAX_ARGS);
         goto EXPAND_FAILED;
         }
-      sub_arg[i] = expand_string_internal(s+1, TRUE, &s, FALSE);
+      sub_arg[i] = expand_string_internal(s+1, TRUE, &s, skipping);
       if (sub_arg[i++] == NULL) goto EXPAND_FAILED;
       while (isspace((uschar)*s)) s++;
       if (*s++ != '}') goto EXPAND_FAILED_CURLY;
@@ -1844,6 +1852,10 @@ while (*s != 0)
     if (*s++ != '}') goto EXPAND_FAILED_CURLY;
     sub_arg[i] = 0;
 
+    /* If skipping, we don't actually do anything */
+
+    if (skipping) continue;
+
     /* Start the interpreter if necessary */
 
     if (!opt_perl_started)
@@ -1904,7 +1916,7 @@ while (*s != 0)
     int o2m;
     char *sub[3];
 
-    switch(read_subs(sub, 3, &s))
+    switch(read_subs(sub, 3, &s, skipping))
       {
       case 1: goto EXPAND_FAILED_CURLY;
       case 2: goto EXPAND_FAILED;
@@ -1941,7 +1953,7 @@ while (*s != 0)
     int save_expand_nmax =
       save_expand_strings(save_expand_nstring, save_expand_nlength);
 
-    switch(read_subs(sub, 3, &s))
+    switch(read_subs(sub, 3, &s, skipping))
       {
       case 1: goto EXPAND_FAILED_CURLY;
       case 2: goto EXPAND_FAILED;
@@ -2057,7 +2069,7 @@ while (*s != 0)
       while (isspace((uschar)*s)) s++;
       if (*s == '{')
         {
-        sub[i] = expand_string_internal(s+1, TRUE, &s, FALSE);
+        sub[i] = expand_string_internal(s+1, TRUE, &s, skipping);
         if (sub[i] == NULL) goto EXPAND_FAILED;
         if (*s++ != '}') goto EXPAND_FAILED_CURLY;
 
@@ -2087,9 +2099,10 @@ while (*s != 0)
       else goto EXPAND_FAILED_CURLY;
       }
 
-    /* Extract either the numbered or the keyed substring into $value. */
+    /* Extract either the numbered or the keyed substring into $value. If
+    skipping, just pretend the extraction failed. */
 
-    lookup_value = (field_number >= 0)?
+    lookup_value = skipping? NULL : (field_number >= 0)?
       expand_gettokened(field_number, sub[1], sub[2]) :
       expand_getkeyed(sub[0], sub[1]);
 
@@ -2123,7 +2136,7 @@ while (*s != 0)
 
   if (*s == ':')
     {
-    char *sub = expand_string_internal(s+1, TRUE, &s, FALSE);
+    char *sub = expand_string_internal(s+1, TRUE, &s, skipping);
     if (sub == NULL) goto EXPAND_FAILED;
     s++;
 
@@ -2131,7 +2144,7 @@ while (*s != 0)
 
     if (strcmp(name, "expand") == 0)
       {
-      char *expanded = expand_string_internal(sub, FALSE, NULL, FALSE);
+      char *expanded = expand_string_internal(sub, FALSE, NULL, skipping);
       if (expanded == NULL)
         {
         expand_string_message =
@@ -2165,6 +2178,21 @@ while (*s != 0)
       continue;
       }
 
+    /* md5 yields the md5 hash */
+
+    if (strcmp(name, "md5") == 0)
+      {
+      md5 base;
+      uschar digest[16];
+      int j;
+      char st[33];
+      md5_start(&base);
+      md5_end(&base, sub, (int)strlen(sub), digest);
+      for(j = 0; j < 16; j++) sprintf(st+2*j, "%02x", digest[j]);
+      yield = string_cat(yield, &size, &ptr, st, (int)strlen(st));
+      continue;
+      }
+
     /* mask applies a mask to an IP address; for example the result of
     ${mask:131.111.10.206/28} is 131.111.10.192/28. */
 
@@ -2517,7 +2545,8 @@ terminating brace. */
 if (ket_ends && *s == 0)
   {
   expand_string_message = malformed_header?
-    "missing } at end of string - could be header name not terminated by colon" :
+    "missing } at end of string - could be header name not terminated by colon"
+    :
     "missing } at end of string";
   goto EXPAND_FAILED;
   }
@@ -2540,7 +2569,10 @@ to update the pointer to the terminator, for cases of nested calls with "fail".
 */
 
 EXPAND_FAILED_CURLY:
-expand_string_message = "missing or misplaced { or }";
+expand_string_message = malformed_header?
+  "missing or misplaced { or } - could be header name not terminated by colon"
+  :
+  "missing or misplaced { or }";
 
 EXPAND_FAILED:
 if (left != NULL) *left = s;
@@ -2561,6 +2593,7 @@ char *
 expand_string(char *string)
 {
 search_find_defer = FALSE;
+malformed_header = FALSE;
 return (strpbrk(string, "$\\") == NULL)? string :
   expand_string_internal(string, FALSE, NULL, FALSE);
 }
