@@ -2099,13 +2099,22 @@ while (commands != NULL)
 
     /* Freeze and fail are available only when there is somewhere to pass
     back the action. These commands are rejected at parse time otherwise.
-    The message must be placed in the MAIN storage pool, since it lives
-    on beyond the running of the filter. */
+    Ensure the message length is not too ridiculous (can become so from, e.g.
+    the inclusion of message header lines). And force printing characters. */
 
     case fail_command:
     case freeze_command:
     filter_fmsg = expargs[0];
-    if (filter_fmsg == NULL && filter_test != NULL) filter_fmsg = "";
+    if (filter_fmsg == NULL)
+      {
+      if (filter_test != NULL) filter_fmsg = "";
+      }
+    else
+      {
+      if (strlen(filter_fmsg) > 1024)
+        strcpy(filter_fmsg + 1000, " ... (truncated)");
+      filter_fmsg = string_printing(filter_fmsg);
+      }
 
     if (commands->command == fail_command)
       {
@@ -2285,6 +2294,9 @@ while (commands != NULL)
       addr->reply->to = string_copy(to);
       addr->reply->file_expand =
         commands->args[mailarg_index_expand] != NULL;
+      addr->reply->forbid_exists = expand_forbid_exists;
+      addr->reply->forbid_lookup = expand_forbid_lookup;
+      addr->reply->forbid_perl = expand_forbid_perl;
       addr->reply->return_message =
         commands->args[mailarg_index_return] != NULL;
 
