@@ -1164,8 +1164,9 @@ set to NULL, fill in its IP address from the DNS. If it is multi-homed, create
 additional host items for the additional addresses, copying all the other
 fields, and randomizing the order.
 
-On IPv6 systems, AAAA records are sought first, and then A records are sought
-as well.
+On IPv6 systems, A6 records are sought first (but only if support for A6 is
+configured - they may never become mainstream), then AAAA records are sought,
+and finally A records are sought as well.
 
 The host name may be changed if the DNS returns a different name - e.g. fully
 qualified or changed via CNAME. If fully_qualified_name is not NULL, dns_lookup
@@ -1215,15 +1216,21 @@ if (allow_ip && string_is_ip_address(host->name, NULL) != 0)
   return HOST_FOUND;
   }
 
-/* On an IPv6 system, go round the loop three times, looking for A6 and AAAA
-records the first two times, unless ipv4_address_lookup is set. On an IPv4
-system, go round the loop once only, looking only for A records. */
+/* On an IPv6 system, go round the loop up to three times, looking for A6 and
+AAAA records the first two times, unless ipv4_address_lookup is set. However,
+since A6 records look like being abandoned, support them only if explicitly
+configured to do so. On an IPv4 system, go round the loop once only, looking
+only for A records. */
 
 #if HAVE_IPV6
-i = ipv4_address_lookup? 0 : 2;
-#else
+  #ifdef SUPPORT_A6
+  i = ipv4_address_lookup? 0 : 2;
+  #else
+  i = ipv4_address_lookup? 0 : 1;
+  #endif  /* SUPPORT_A6 */
+#else   /* HAVE_IPV6 */
 i = 0;
-#endif
+#endif  /* HAVE_IPV6 */
 
 for (; i >= 0; i--)
   {
@@ -1266,8 +1273,6 @@ for (; i >= 0; i--)
     {
     if (rr->type == type)
       {
-      /* dns_address *da = dns_address_from_rr(&dnsa, rr); */
-
       dns_address *da;
       da = dns_address_from_rr(&dnsa, rr);
 
@@ -1364,7 +1369,7 @@ for (; i >= 0; i--)
     }
   }
 
-/* Control gets here only if the third lookup (the A record) succeeded.
+/* Control gets here only if the final lookup (the A record) succeeded.
 However, the address may not be filled in if it was ignored. */
 
 return (host->address == NULL)? HOST_IGNORED : HOST_FOUND;
@@ -1637,7 +1642,9 @@ for (rr = dns_next_rr(&dnsa, &dnss, RESET_ADDITIONAL);
   if (rr->type != T_A
   #if HAVE_IPV6
     && rr->type != T_AAAA
+    #ifdef SUPPORT_A6
     && rr->type != T_A6
+    #endif
   #endif
     ) continue;
 
@@ -1660,7 +1667,7 @@ for (rr = dns_next_rr(&dnsa, &dnss, RESET_ADDITIONAL);
           verify_check_this_host(&ignore_target_hosts, FALSE, h->name,
           CS da->address, NULL))
       {
-      DEBUG(8) debug_printf("ignored host %s [%s]\n", host->name, da->address);
+      DEBUG(8) debug_printf("ignored host %s [%s]\n", h->name, da->address);
       status = hstatus_unusable;
       why = hwhy_ignored;
       }
@@ -1794,6 +1801,7 @@ if (ignore_target_hosts != NULL)
       {
       if (h != last)              /* First is not last, overwrite it */
         {
+        if (h->next == last) last = h;
         *h = *(h->next);          /* and reprocess it. */
         goto REDO;                /* C should have redo, like Perl */
         }
@@ -1946,7 +1954,11 @@ if (rc == DNS_SUCCEED)
 	   rr = dns_next_rr(&dnsa, &dnss, RESET_NEXT))
 	{
         #if HAVE_IPV6
-        if (rr->type == T_A || rr->type == T_A6 || rr->type == T_AAAA)
+          #ifdef SUPPORT_A6
+          if (rr->type == T_A || rr->type == T_A6 || rr->type == T_AAAA)
+          #else
+          if (rr->type == T_A || rr->type == T_AAAA)
+          #endif
         #else
         if (rr->type == T_A)
         #endif
