@@ -55,9 +55,11 @@ Returns:         nothing
 static void
 write_syslog(int priority, char *s)
 {
-int pass;
+int len, pass;
 int linecount = 0;
-int len = (int)strlen(s);
+
+if (!syslog_timestamp) s += 20;
+len = (int)strlen(s);
 
 #ifndef NO_OPENLOG
 if (!syslog_open)
