@@ -471,9 +471,6 @@ while (argc > 1 && argv[op][0] == '-')
     printf("Usage: pcretest [-d] [-i] [-p] [-s] [-t] [<input> [<output>]]\n");
     printf("  -d   debug: show compiled code; implies -i\n"
            "  -i   show information about compiled pattern\n"
-#if !defined NOPOSIX            
-           "  -p   use POSIX interface\n"
-#endif            
            "  -s   output store information\n"
            "  -t   time compilation and execution\n");
     return 1;
