@@ -459,6 +459,9 @@ typedef struct reply_item {
   char *text;                    /* text string body */
   char *file;                    /* file body */
   BOOL  file_expand;             /* expand the body */
+  BOOL  forbid_exists;           /* things not allowed in expansion */
+  BOOL  forbid_lookup;
+  BOOL  forbid_perl;
   char *logfile;                 /* file to keep a log in */
   char *oncelog;                 /* file to keep records in for once only */
   time_t once_repeat;            /* time to repeat "once only" */
