@@ -693,14 +693,12 @@ SMTP dot-handling. */
 if (tblock->local_smtp != local_smtp_off)
   {
   address_item *a;
-  char *sender = (addr->errors_address != NULL)?
-    addr->errors_address : sender_address;
 
   if (tblock->bsmtp_helo &&
       !transport_write_string(fd_in, "HELO %s%s", primary_hostname, eol))
     goto END_WRITE;
 
-  if (!transport_write_string(fd_in, "MAIL FROM:<%s>%s", sender, eol))
+  if (!transport_write_string(fd_in, "MAIL FROM:<%s>%s", return_path, eol))
     goto END_WRITE;
 
   for (a = addr; a != NULL; a = a->next)
