@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2000 */
+/* Copyright (c) University of Cambridge 1995 - 2001 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 #include "../exim.h"
@@ -45,6 +45,8 @@ optionlist smtp_transport_options[] = {
       (void *)(offsetof(smtp_transport_options_block, final_timeout)) },
   { "gethostbyname",        opt_bool,
       (void *)(offsetof(smtp_transport_options_block, gethostbyname)) },
+  { "helo_data",            opt_stringptr,
+      (void *)(offsetof(smtp_transport_options_block, helo_data)) },
   { "hosts",                opt_stringptr,
       (void *)(offsetof(smtp_transport_options_block, hosts)) },
   #ifdef SUPPORT_TLS
@@ -109,41 +111,42 @@ int smtp_transport_options_count =
 /* Default private options block for the smtp transport. */
 
 smtp_transport_options_block smtp_transport_option_defaults = {
-  NULL,            /* hosts */
-  NULL,            /* fallback_hosts */
-  NULL,            /* hostlist */
-  NULL,            /* fallback_hostlist */
-  NULL,            /* interface */
-  NULL,            /* service (i.e. port) */
-  "smtp",          /* protocol */
-  NULL,            /* mx_domains */
-  NULL,            /* serialize_hosts */
-  NULL,            /* authenticate_hosts */
-  NULL,            /* hosts_require_tls */
-  NULL,            /* hosts_avoid_tls */
-  500,             /* batch_max */
-  5*60,            /* command_timeout */
-  5*60,            /* connect_timeout; shorter system default overrides */
-  5*60,            /* data timeout */
-  10*60,           /* final timeout */
-  1024,            /* size_addition */
-  0,               /* max_rcpt */
-  5,               /* hosts_max_try */
-  FALSE,           /* allow_localhost */
-  FALSE,           /* gethostbyname */
-  TRUE,            /* dns_qualify_single */
-  FALSE,           /* dns_search_parents */
-  TRUE,            /* delay_after_cutoff */
-  FALSE,           /* hosts_override */
-  FALSE,           /* hosts_randomize */
-  TRUE,            /* keepalive */
-  FALSE,           /* max_rcpt_serialize */
-  TRUE             /* retry_include_ip_address */
+  NULL,                /* hosts */
+  NULL,                /* fallback_hosts */
+  NULL,                /* hostlist */
+  NULL,                /* fallback_hostlist */
+  "$primary_hostname", /* helo_data */
+  NULL,                /* interface */
+  NULL,                /* service (i.e. port) */
+  "smtp",              /* protocol */
+  NULL,                /* mx_domains */
+  NULL,                /* serialize_hosts */
+  NULL,                /* authenticate_hosts */
+  NULL,                /* hosts_require_tls */
+  NULL,                /* hosts_avoid_tls */
+  500,                 /* batch_max */
+  5*60,                /* command_timeout */
+  5*60,                /* connect_timeout; shorter system default overrides */
+  5*60,                /* data timeout */
+  10*60,               /* final timeout */
+  1024,                /* size_addition */
+  0,                   /* max_rcpt */
+  5,                   /* hosts_max_try */
+  FALSE,               /* allow_localhost */
+  FALSE,               /* gethostbyname */
+  TRUE,                /* dns_qualify_single */
+  FALSE,               /* dns_search_parents */
+  TRUE,                /* delay_after_cutoff */
+  FALSE,               /* hosts_override */
+  FALSE,               /* hosts_randomize */
+  TRUE,                /* keepalive */
+  FALSE,               /* max_rcpt_serialize */
+  TRUE                 /* retry_include_ip_address */
   #ifdef SUPPORT_TLS
- ,NULL,            /* tls_certificate */
-  NULL,            /* tls_privatekey */
-  NULL,            /* tls_verify_certificates */
-  NULL             /* tls_verify_ciphers */
+ ,NULL,                /* tls_certificate */
+  NULL,                /* tls_privatekey */
+  NULL,                /* tls_verify_certificates */
+  NULL                 /* tls_verify_ciphers */
   #endif
 };
 
@@ -683,8 +686,8 @@ Returns:          OK    - the connection was made and the delivery attempted;
                           while setting up the SMTP session, or there was a
                           non-message-specific error, such as a timeout.
                   ERROR - a filter command is specified for this transport,
-                          and there was a problem setting it up; OR
-                          add_headers is specified for this transport, and
+                          and there was a problem setting it up; OR helo_data
+                          or add_headers is specified for this transport, and
                           the string failed to expand
 */
 
@@ -743,6 +746,8 @@ the initial interaction and HELO/EHLO/LHLO. */
 
 if (continue_hostname == NULL)
   {
+  char *helo_data;
+
   #ifdef SUPPORT_TLS
   tls_offered = FALSE;
   #endif
@@ -790,8 +795,22 @@ goto SEND_QUIT;
   mailers use upper case for some reason (the RFC is quite clear about case
   independence) so, for peace of mind, I gave in. */
 
+  deliver_host = host->name;
+  deliver_host_address = host->address;
+  helo_data = expand_string(ob->helo_data);
+  deliver_host = deliver_host_address = NULL;
+
+  if (helo_data == NULL)
+    {
+    char *message = string_sprintf("failed to expand helo_data: %s",
+      expand_string_message);
+    set_errno(addrlist, 0, message, DEFER);
+    yield = ERROR;
+    goto SEND_QUIT;
+    }
+
   if (!smtp_write_command(&outblock, FALSE, "%s %s\r\n", lmtp? "LHLO" : "EHLO",
-        primary_hostname))
+        helo_data))
     goto SEND_FAILED;
 
   if (!smtp_read_response(&inblock, buffer, sizeof(buffer), '2',
